@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import logging
 import sys
 from errno import EINVAL, EPIPE
@@ -37,33 +39,51 @@ class TestStdoutStderr:
         monkeypatch.setattr("streamlink_cli.main.log_current_arguments", Mock())
 
     # noinspection PyUnresolvedReferences
-    @pytest.mark.parametrize(("argv", "stream"), [
-        pytest.param([], "stdout", id="default"),
-        pytest.param(["--stdout"], "stderr", id="--stdout"),
-        pytest.param(["--output=file"], "stdout", id="--output=file"),
-        pytest.param(["--output=-"], "stderr", id="--output=-"),
-        pytest.param(["--record=file"], "stdout", id="--record=file"),
-        pytest.param(["--record=-"], "stderr", id="--record=-"),
-        pytest.param(["--record-and-pipe=file"], "stderr", id="--record-and-pipe=file"),
-    ], indirect=["argv"])
-    def test_streams(self, capsys: pytest.CaptureFixture, parser: ArgumentParser, argv: list, stream: str):
+    @pytest.mark.parametrize(
+        ("argv", "stream"),
+        [
+            pytest.param([], "stdout", id="default"),
+            pytest.param(["--quiet"], None, id="--quiet"),
+            pytest.param(["--stdout"], "stderr", id="--stdout"),
+            pytest.param(["--output=file"], "stdout", id="--output=file"),
+            pytest.param(["--output=-"], "stderr", id="--output=-"),
+            pytest.param(["--record=file"], "stdout", id="--record=file"),
+            pytest.param(["--record=-"], "stderr", id="--record=-"),
+            pytest.param(["--record-and-pipe=file"], "stderr", id="--record-and-pipe=file"),
+        ],
+        indirect=["argv"],
+    )
+    def test_streams(self, capsys: pytest.CaptureFixture, parser: ArgumentParser, argv: list, stream: str | None):
         streamlink_cli.main.setup(parser)
 
         rootlogger = logging.getLogger("streamlink")
         clilogger = streamlink_cli.main.log
-        streamobj = getattr(sys, stream)
+        streamobj = {
+            None: None,
+            "stdout": sys.stdout,
+            "stderr": sys.stderr,
+        }.get(stream)
 
+        assert streamlink_cli.main.console.console_output is streamobj
+        assert streamlink_cli.main.console.file_output is None
         assert clilogger.parent is rootlogger
-        assert isinstance(rootlogger.handlers[0], logging.StreamHandler)
-        assert rootlogger.handlers[0].stream is streamobj
-        assert streamlink_cli.main.console.output is streamobj
+        if stream is None:
+            assert not rootlogger.handlers
+        else:
+            handler = rootlogger.handlers[0]
+            assert isinstance(handler, logging.StreamHandler)
+            assert handler.stream is streamobj
 
-    @pytest.mark.parametrize(("argv", "stdout", "stderr"), [
-        pytest.param([], "[cli][info] a\n[test_main_logging][error] b\nerror: c\n", "", id="no-pipe-no-json"),
-        pytest.param(["--json"], "{\n  \"error\": \"c\"\n}\n", "", id="no-pipe-json"),
-        pytest.param(["--stdout"], "", "[cli][info] a\n[test_main_logging][error] b\nerror: c\n", id="pipe-no-json"),
-        pytest.param(["--stdout", "--json"], "", "{\n  \"error\": \"c\"\n}\n", id="pipe-json"),
-    ], indirect=["argv"])
+    @pytest.mark.parametrize(
+        ("argv", "stdout", "stderr"),
+        [
+            pytest.param([], "[cli][info] a\n[test_main_logging][error] b\nerror: c\n", "", id="no-pipe-no-json"),
+            pytest.param(["--json"], '{\n  "error": "c"\n}\n', "", id="no-pipe-json"),
+            pytest.param(["--stdout"], "", "[cli][info] a\n[test_main_logging][error] b\nerror: c\n", id="pipe-no-json"),
+            pytest.param(["--stdout", "--json"], "", '{\n  "error": "c"\n}\n', id="pipe-json"),
+        ],
+        indirect=["argv"],
+    )
     def test_output(
         self,
         monkeypatch: pytest.MonkeyPatch,
@@ -88,12 +108,60 @@ class TestStdoutStderr:
         assert out == stdout
         assert err == stderr
 
-    def test_no_stdout(self, monkeypatch: pytest.MonkeyPatch):
-        monkeypatch.setattr("sys.stdout", None)
+    @pytest.mark.parametrize(
+        ("missing_stdio", "expected_stdout", "expected_stderr"),
+        [
+            pytest.param(
+                [],
+                "[cli][info] a\n[test_main_logging][error] b\nerror: c\n",
+                "",
+                id="none-missing",
+            ),
+            pytest.param(
+                ["sys.stdout"],
+                "",
+                "[cli][info] a\n[test_main_logging][error] b\nerror: c\n",
+                id="missing-stdout",
+            ),
+            pytest.param(
+                ["sys.stderr"],
+                "[cli][info] a\n[test_main_logging][error] b\nerror: c\n",
+                "",
+                id="missing-stderr",
+            ),
+            pytest.param(
+                ["sys.stdout", "sys.stderr"],
+                "",
+                "",
+                id="missing-stdout-and-stderr",
+            ),
+        ],
+    )
+    def test_missing_stdio(
+        self,
+        monkeypatch: pytest.MonkeyPatch,
+        capsys: pytest.CaptureFixture,
+        missing_stdio: list[str],
+        expected_stdout: str,
+        expected_stderr: str,
+    ):
+        def run(_parser):
+            childlogger = logging.getLogger("streamlink.test_main_logging")
+            streamlink_cli.main.log.info("a")
+            childlogger.error("b")
+            raise StreamlinkCLIError("c")
+
+        monkeypatch.setattr("streamlink_cli.main.run", run)
+        for item in missing_stdio:
+            monkeypatch.setattr(item, None)
 
         with pytest.raises(SystemExit) as excinfo:
             streamlink_cli.main.main()
-        assert excinfo.value.code == 0
+        assert excinfo.value.code == 1
+
+        out, err = capsys.readouterr()
+        assert out == expected_stdout
+        assert err == expected_stderr
 
     @pytest.mark.parametrize(
         "errno",
@@ -178,10 +246,14 @@ class TestStdoutStderr:
 class TestInfos:
     # noinspection PyTestParametrized
     @pytest.mark.posix_only()
-    @pytest.mark.parametrize(("_euid", "logs"), [
-        pytest.param(1000, [], id="user"),
-        pytest.param(0, [("cli", "info", "streamlink is running as root! Be careful!")], id="root"),
-    ], indirect=["_euid"])
+    @pytest.mark.parametrize(
+        ("_euid", "logs"),
+        [
+            pytest.param(1000, [], id="user"),
+            pytest.param(0, [("cli", "info", "streamlink is running as root! Be careful!")], id="root"),
+        ],
+        indirect=["_euid"],
+    )
     def test_log_root_warning(
         self,
         monkeypatch: pytest.MonkeyPatch,
@@ -195,56 +267,60 @@ class TestInfos:
         streamlink_cli.main.setup(parser)
         assert [(record.name, record.levelname, record.message) for record in caplog.records] == logs
 
-    @pytest.mark.parametrize(("argv", "platform", "logs"), [
-        pytest.param(
-            ["--loglevel", "info"],
-            "linux",
-            [],
-            id="non-debug-loglevel",
-        ),
-        pytest.param(
-            ["--loglevel", "debug"],
-            "darwin",
-            [
-                ("cli", "debug", "OS:         macOS 0.0.0"),
-                ("cli", "debug", "Python:     PYTHON_VERSION"),
-                ("cli", "debug", "OpenSSL:    OPENSSL_VERSION"),
-                ("cli", "debug", "Streamlink: STREAMLINK_VERSION"),
-                ("cli", "debug", "Dependencies:"),
-                ("cli", "debug", " foo: 1.2.3"),
-                ("cli", "debug", " bar-baz: 2.0.0"),
-            ],
-            id="darwin",
-        ),
-        pytest.param(
-            ["--loglevel", "debug"],
-            "win32",
-            [
-                ("cli", "debug", "OS:         Windows 0.0.0"),
-                ("cli", "debug", "Python:     PYTHON_VERSION"),
-                ("cli", "debug", "OpenSSL:    OPENSSL_VERSION"),
-                ("cli", "debug", "Streamlink: STREAMLINK_VERSION"),
-                ("cli", "debug", "Dependencies:"),
-                ("cli", "debug", " foo: 1.2.3"),
-                ("cli", "debug", " bar-baz: 2.0.0"),
-            ],
-            id="win32",
-        ),
-        pytest.param(
-            ["--loglevel", "debug"],
-            "linux",
-            [
-                ("cli", "debug", "OS:         linux"),
-                ("cli", "debug", "Python:     PYTHON_VERSION"),
-                ("cli", "debug", "OpenSSL:    OPENSSL_VERSION"),
-                ("cli", "debug", "Streamlink: STREAMLINK_VERSION"),
-                ("cli", "debug", "Dependencies:"),
-                ("cli", "debug", " foo: 1.2.3"),
-                ("cli", "debug", " bar-baz: 2.0.0"),
-            ],
-            id="linux",
-        ),
-    ], indirect=["argv"])
+    @pytest.mark.parametrize(
+        ("argv", "platform", "logs"),
+        [
+            pytest.param(
+                ["--loglevel", "info"],
+                "linux",
+                [],
+                id="non-debug-loglevel",
+            ),
+            pytest.param(
+                ["--loglevel", "debug"],
+                "darwin",
+                [
+                    ("cli", "debug", "OS:         macOS 0.0.0"),
+                    ("cli", "debug", "Python:     PYTHON_VERSION"),
+                    ("cli", "debug", "OpenSSL:    OPENSSL_VERSION"),
+                    ("cli", "debug", "Streamlink: STREAMLINK_VERSION"),
+                    ("cli", "debug", "Dependencies:"),
+                    ("cli", "debug", " bar-baz: 2.0.0"),
+                    ("cli", "debug", " foo: 1.2.3"),
+                ],
+                id="darwin",
+            ),
+            pytest.param(
+                ["--loglevel", "debug"],
+                "win32",
+                [
+                    ("cli", "debug", "OS:         Windows 0.0.0"),
+                    ("cli", "debug", "Python:     PYTHON_VERSION"),
+                    ("cli", "debug", "OpenSSL:    OPENSSL_VERSION"),
+                    ("cli", "debug", "Streamlink: STREAMLINK_VERSION"),
+                    ("cli", "debug", "Dependencies:"),
+                    ("cli", "debug", " bar-baz: 2.0.0"),
+                    ("cli", "debug", " foo: 1.2.3"),
+                ],
+                id="win32",
+            ),
+            pytest.param(
+                ["--loglevel", "debug"],
+                "linux",
+                [
+                    ("cli", "debug", "OS:         linux"),
+                    ("cli", "debug", "Python:     PYTHON_VERSION"),
+                    ("cli", "debug", "OpenSSL:    OPENSSL_VERSION"),
+                    ("cli", "debug", "Streamlink: STREAMLINK_VERSION"),
+                    ("cli", "debug", "Dependencies:"),
+                    ("cli", "debug", " bar-baz: 2.0.0"),
+                    ("cli", "debug", " foo: 1.2.3"),
+                ],
+                id="linux",
+            ),
+        ],
+        indirect=["argv"],
+    )
     def test_log_current_versions(
         self,
         monkeypatch: pytest.MonkeyPatch,
@@ -266,7 +342,12 @@ class TestInfos:
 
         mock_importlib_metadata = Mock()
         mock_importlib_metadata.PackageNotFoundError = FakePackageNotFoundError
-        mock_importlib_metadata.requires.return_value = ["foo>1", "bar-baz==2", "qux~=3"]
+        mock_importlib_metadata.requires.return_value = [
+            "foo>1 ; python_version>='3.13'",
+            "foo<=1 ; python_version<'3.13'",
+            "bar-baz==2",
+            "qux~=3",
+        ]
         mock_importlib_metadata.version.side_effect = version
 
         monkeypatch.setattr("importlib.metadata", mock_importlib_metadata)
@@ -286,35 +367,39 @@ class TestInfos:
         assert mock_importlib_metadata.requires.call_args_list == ([call("streamlink")] if logs else [])
         assert [(record.name, record.levelname, record.message) for record in caplog.records] == logs
 
-    @pytest.mark.parametrize(("argv", "logs"), [
-        pytest.param(
-            ["--loglevel", "info"],
-            [],
-            id="non-debug-loglevel",
-        ),
-        pytest.param(
-            [
-                "--loglevel",
-                "debug",
-                "-p",
-                "custom",
-                "--testplugin-bool",
-                "--testplugin-password=secret",
-                "test.se/channel",
-                "best,worst",
-            ],
-            [
-                ("cli", "debug", "Arguments:"),
-                ("cli", "debug", " url=test.se/channel"),
-                ("cli", "debug", " stream=['best', 'worst']"),
-                ("cli", "debug", " --loglevel=debug"),
-                ("cli", "debug", " --player=custom"),
-                ("cli", "debug", " --testplugin-bool=True"),
-                ("cli", "debug", " --testplugin-password=********"),
-            ],
-            id="arguments",
-        ),
-    ], indirect=["argv"])
+    @pytest.mark.parametrize(
+        ("argv", "logs"),
+        [
+            pytest.param(
+                ["--loglevel", "info"],
+                [],
+                id="non-debug-loglevel",
+            ),
+            pytest.param(
+                [
+                    "--loglevel",
+                    "debug",
+                    "-p",
+                    "custom",
+                    "--testplugin-bool",
+                    "--testplugin-password=secret",
+                    "test.se/channel",
+                    "best,worst",
+                ],
+                [
+                    ("cli", "debug", "Arguments:"),
+                    ("cli", "debug", " url=test.se/channel"),
+                    ("cli", "debug", " stream=['best', 'worst']"),
+                    ("cli", "debug", " --loglevel=debug"),
+                    ("cli", "debug", " --player=custom"),
+                    ("cli", "debug", " --testplugin-bool=True"),
+                    ("cli", "debug", " --testplugin-password=********"),
+                ],
+                id="arguments",
+            ),
+        ],
+        indirect=["argv"],
+    )
     def test_log_current_arguments(
         self,
         monkeypatch: pytest.MonkeyPatch,
@@ -402,38 +487,64 @@ class TestLogfile:
         return tmp_path
 
     # noinspection PyUnresolvedReferences
-    @pytest.mark.parametrize(("argv", "stdout", "stderr"), [
-        pytest.param(
-            [],
-            "[cli][info] a\nb\n",
-            "",
-            id="no-logfile",
-        ),
-        pytest.param(
-            ["--logfile=file", "--loglevel=none"],
-            "b\n",
-            "",
-            id="logfile-loglevel-none",
-        ),
-    ], indirect=["argv"])
+    @pytest.mark.parametrize(
+        ("argv", "stream", "stdout", "stderr"),
+        [
+            pytest.param(
+                [],
+                "stdout",
+                "[cli][info] a\nb\n",
+                "",
+                id="no-logfile",
+            ),
+            pytest.param(
+                ["--logfile=file", "--loglevel=none"],
+                "stdout",
+                "b\n",
+                "",
+                id="logfile-loglevel-none",
+            ),
+            pytest.param(
+                ["--logfile=file", "--quiet"],
+                None,
+                "",
+                "",
+                id="logfile-quiet",
+            ),
+        ],
+        indirect=["argv"],
+    )
     def test_no_logfile(
         self,
         monkeypatch: pytest.MonkeyPatch,
         capsys: pytest.CaptureFixture,
         parser: ArgumentParser,
         argv: list,
+        stream: str | None,
         stdout: str,
         stderr: str,
     ):
         mock_open = Mock()
         monkeypatch.setattr("builtins.open", mock_open)
 
+        streamobj = {
+            None: None,
+            "stdout": sys.stdout,
+            "stderr": sys.stderr,
+        }.get(stream)
+
         streamlink_cli.main.setup(parser)
 
+        assert streamlink_cli.main.console.console_output is streamobj
+        assert streamlink_cli.main.console.file_output is None
+
         rootlogger = logging.getLogger("streamlink")
-        assert isinstance(rootlogger.handlers[0], logging.StreamHandler)
-        assert rootlogger.handlers[0].stream is sys.stdout
-        assert streamlink_cli.main.console.output is sys.stdout
+        if stream is None:
+            assert not rootlogger.handlers
+        else:
+            handler = rootlogger.handlers[0]
+            assert isinstance(handler, logging.StreamHandler)
+            assert handler.stream is streamobj
 
         streamlink_cli.main.log.info("a")
         streamlink_cli.main.console.msg("b")
@@ -443,26 +554,33 @@ class TestLogfile:
         assert err == stderr
 
     # noinspection PyUnresolvedReferences
-    @pytest.mark.parametrize(("argv", "path", "content"), [
-        pytest.param(
-            ["--logfile=path/to/logfile"],
-            Path("path", "to", "logfile"),
-            "[cli][info] a\nb\n",
-            id="logfile-path-resolve",
-        ),
-        pytest.param(
-            ["--logfile=~/path/to/logfile"],
-            Path("user", "path", "to", "logfile"),
-            "[cli][info] a\nb\n",
-            id="logfile-path-expanduser",
-        ),
-        pytest.param(
-            ["--logfile=-"],
-            Path("user", "logs", "2000-01-01_12-34-56.log"),
-            "[cli][info] a\nb\n",
-            id="logfile-auto",
-        ),
-    ], indirect=["argv"])
+    @pytest.mark.parametrize(
+        ("argv", "path", "logcontent", "filecontent"),
+        [
+            pytest.param(
+                ["--logfile=path/to/logfile"],
+                Path("path", "to", "logfile"),
+                "[cli][info] a\nb\n",
+                "b\n",
+                id="logfile-path-resolve",
+            ),
+            pytest.param(
+                ["--logfile=~/path/to/logfile"],
+                Path("user", "path", "to", "logfile"),
+                "[cli][info] a\nb\n",
+                "b\n",
+                id="logfile-path-expanduser",
+            ),
+            pytest.param(
+                ["--logfile=-"],
+                Path("user", "logs", "2000-01-01_12-34-56.log"),
+                "[cli][info] a\nb\n",
+                "b\n",
+                id="logfile-auto",
+            ),
+        ],
+        indirect=["argv"],
+    )
     def test_logfile(
         self,
         monkeypatch: pytest.MonkeyPatch,
@@ -471,7 +589,8 @@ class TestLogfile:
         logpath: Path,
         argv: list,
         path: str,
-        content: str,
+        logcontent: str,
+        filecontent: str,
     ):
         abspath = Path().resolve() / path
 
@@ -486,19 +605,49 @@ class TestLogfile:
         assert isinstance(rootlogger.handlers[0], logging.FileHandler)
         assert rootlogger.handlers[0].baseFilename == str(abspath)
         assert rootlogger.handlers[0].stream is streamobj
-        assert streamlink_cli.main.console.output is streamobj
+        assert streamlink_cli.main.console.console_output is not streamobj
+        assert streamlink_cli.main.console.file_output is streamobj
 
         streamlink_cli.main.log.info("a")
         streamlink_cli.main.console.msg("b")
         out, err = capsys.readouterr()
-        # TODO: py38 support end: replace conditional assertion
-        assert mock_open.call_args_list == (
-            [call(str(abspath), "a", encoding="utf-8")]
-            if sys.version_info < (3, 9) else
-            [call(str(abspath), "a", encoding="utf-8", errors=None)]
-        )
-        assert streamobj.getvalue() == content
-        assert out == ""
+        assert mock_open.call_args_list == [call(str(abspath), "a", encoding="utf-8", errors=None)]
+        assert streamobj.getvalue() == logcontent
+        assert out == filecontent
+        assert err == ""
+
+    @pytest.mark.parametrize("argv", [pytest.param(["--logfile=tty"], id="tty")], indirect=["argv"])
+    def test_logfile_isatty(
+        self,
+        monkeypatch: pytest.MonkeyPatch,
+        capsys: pytest.CaptureFixture,
+        argv: list,
+        parser: ArgumentParser,
+    ):
+        abspath = str(Path().resolve() / "tty")
+
+        streamobj = StringIO()
+        streamobj.isatty = lambda: True  # type: ignore[method-assign]
+
+        mock_open = Mock(return_value=streamobj)
+        monkeypatch.setattr("builtins.open", mock_open)
+
+        streamlink_cli.main.setup(parser)
+        assert mock_open.call_args_list == [call(abspath, "a", encoding="utf-8", errors=None)]
+
+        rootlogger = logging.getLogger("streamlink")
+        handler = rootlogger.handlers[0]
+        assert isinstance(handler, logging.FileHandler)
+        assert handler.stream is streamobj
+
+        assert streamlink_cli.main.console.console_output is not streamobj
+        assert streamlink_cli.main.console.file_output is None
+
+        streamlink_cli.main.log.info("a")
+        streamlink_cli.main.console.msg("b")
+        out, err = capsys.readouterr()
+        assert streamobj.getvalue() == "[cli][info] a\n"
+        assert out == "b\n"
         assert err == ""
 
 
@@ -519,30 +668,43 @@ class TestPrint:
         return out
 
     def test_usage(self, stdout: str):
-        assert stdout == dedent("""
-            usage: streamlink [OPTIONS] <URL> [STREAM]
+        assert (
+            stdout
+            == dedent("""
+                usage: streamlink [OPTIONS] <URL> [STREAM]
 
-            Use -h/--help to see the available options or read the manual at https://streamlink.github.io
-        """).lstrip()
+                Use -h/--help to see the available options or read the manual at https://streamlink.github.io/
+            """).lstrip()
+        )
 
     @pytest.mark.parametrize("argv", [["--help"]], indirect=["argv"])
     def test_help(self, argv: list, stdout: str):
         assert "usage: streamlink [OPTIONS] <URL> [STREAM]" in stdout
-        assert dedent("""
-            Streamlink is a command-line utility that extracts streams from various
-            services and pipes them into a video player of choice.
-        """) in stdout
-        assert dedent("""
-            For more in-depth documentation see:
-              https://streamlink.github.io
+        assert (
+            dedent("""
+                Streamlink is a command-line utility that extracts streams from various
+                services and pipes them into a video player of choice.
+            """)
+            in stdout
+        )
+        assert (
+            dedent("""
+                For more in-depth documentation see:
+                  https://streamlink.github.io/
 
-            Please report broken plugins or bugs to the issue tracker on Github:
-              https://github.com/streamlink/streamlink/issues
-        """) in stdout
+                Please report broken plugins or bugs to the issue tracker on GitHub:
+                  https://github.com/streamlink/streamlink/issues
+            """)
+            in stdout
+        )
 
-    @pytest.mark.parametrize(("argv", "expected"), [
-        pytest.param(["--plugins"], "Available plugins: testplugin\n", id="plugins-no-json"),
-        pytest.param(["--plugins", "--json"], """[\n  "testplugin"\n]\n""", id="plugins-json"),
-    ], indirect=["argv"])
+    @pytest.mark.parametrize(
+        ("argv", "expected"),
+        [
+            pytest.param(["--plugins"], "Available plugins: testplugin\n", id="plugins-no-json"),
+            pytest.param(["--plugins", "--json"], """[\n  "testplugin"\n]\n""", id="plugins-json"),
+        ],
+        indirect=["argv"],
+    )
     def test_plugins(self, argv: list, expected: str, stdout: str):
         assert stdout == expected
